import {
  buildNewSvgImages,
  buildOtherSvgImages,
  buildPlayerAvatars,
  drawAllNewCells,
  drawBackgrounds,
  drawBackgroundTopElements,
  drawFlags,
  drawNewAdditionalStuff,
  images,
} from './western-world.utils';
import { shouldRedraw, getMaxSide } from './western-world.utils.helpers';

if (typeof window !== 'undefined') {
  const template = document.createElement('template');
  template.innerHTML = `
    <style>
      :host {
        display: block;
      }
      canvas {
        width: 100%;
        top: 0;
        left: 0;
        transition: filter 0.5s;
      }
    </style>

    <canvas></canvas>
  `;

  class WesternWorld extends HTMLElement {
    constructor() {
      console.log('WesternWorld constructor...');
      super();
      this._shadowRoot = this.attachShadow({ mode: 'open' });
      this._shadowRoot.appendChild(template.content.cloneNode(true));
      this.$canvas = this._shadowRoot.querySelector('canvas');
    }

    areAllInitialized() {
      return this.language && this._showWesternBackground !== undefined && this._game && this._allCells;
    }

    set language(value) {
      const allInitialized = this.areAllInitialized();
      this._language = value;
      if (allInitialized) {
        this.render();
      }
    }

    set showWesternBackground(value) {
      const allInitialized = this.areAllInitialized();
      this._showWesternBackground = value;
      if (allInitialized) {
        this.render();
      }
    }

    set allCells(value) {
      const allInitialized = this.areAllInitialized();
      this._allCells = value;
      if (allInitialized) {
        this.render();
      }
    }

    set game(value) {
      const allInitialized = this.areAllInitialized();
      const redraw = shouldRedraw(value, this._game);
      this._game = value;
      if (allInitialized && redraw) {
        this.render();
      }
    }

    render() {
      this.fetchImages().then(() => this.drawWesternWorld());
    }

    fetchImages() {
      console.log('fetch images...');
      // get images from cache if exist;
      if (window.cachedImages?.[this._language]) {
        images.svgTags = window.cachedImages?.[this._language]?.svgTags;
        images.svgImagesForCanvas = window.cachedImages?.[this._language]?.svgImagesForCanvas;
        this.dispatchEvent(new CustomEvent('allImagesLoaded', { detail: images }));
        return Promise.resolve();
      }

      images.allImagesProgress.resetLoadingImage();
      Promise.all([
        buildNewSvgImages({
          allCells: this._allCells,
          getMaxSide,
          language: this._language,
          PATH_STROKE_WIDTH_FOR_MASK: this.PATH_STROKE_WIDTH,
          PATH_STROKE_WIDTH_FOR_MASK: this.PATH_STROKE_WIDTH_FOR_MASK,
        }),
        buildOtherSvgImages({ language: _language }),
        buildPlayerAvatars(this.playersIconMap),
      ])
        .then(() => {
          console.log('images fetched...');
          if (!window.cachedImages) {
            window.cachedImages = {};
          }
          if (!window.cachedImages[this._language]) {
            window.cachedImages[this._language] = {};
          }
          window.cachedImages[this._language].svgTags = images.svgTags;
          window.cachedImages[this._language].svgImagesForCanvas = images.svgImagesForCanvas;
          this.dispatchEvent(new CustomEvent('allImagesLoaded', { detail: images }));
        })
        .catch((e) => console.log('something went wrong', e));
    }

    drawWesternWorld() {
      this.$canvas.width = this.width + this.PADDING_LEFT + this.PADDING_RIGHT;
      this.$canvas.height = this.height + this.PADDING_TOP + this.PADDING_BOTTOM;
      const ctx = this.$canvas.getContext('2d');

      if (this._showWesternBackground) {
        this.$canvas.style.background = 'white';
        drawBackgrounds({
          ctx,
          BACKGROUND_X: this.BACKGROUND_X,
          BACKGROUND_Y: this.BACKGROUND_Y,
          BACKGROUND_WIDTH: this.BACKGROUND_WIDTH,
          BACKGROUND_HEIGHT: this.BACKGROUND_HEIGHT,
        });
      } else {
        this.$canvas.style.background = 'transparent';
      }

      // append all new images;
      drawAllNewCells({
        allCells: this._allCells,
        ctx,
        getMaxSide,
        SIZE_X: this.SIZE_X,
        PADDING_LEFT: this.PADDING_LEFT,
        PADDING_TOP: this.PADDING_TOP,
      });

      if (this._showWesternBackground) {
        drawBackgroundTopElements({
          ctx,
          BACKGROUND_X: this.BACKGROUND_X,
          BACKGROUND_Y: this.BACKGROUND_Y,
          BACKGROUND_WIDTH: this.BACKGROUND_WIDTH,
          BACKGROUND_HEIGHT: this.BACKGROUND_HEIGHT,
        });
      }

      drawNewAdditionalStuff({
        ctx,
        SIZE_X: this.SIZE_X,
        PADDING_LEFT: this.PADDING_LEFT,
        PADDING_TOP: this.PADDING_TOP,
      });

      // draw flags;
      drawFlags({
        allCells: this._allCells,
        ctx,
        game: this._game,
        myIdKey: this.myIdKey,
        SIZE_X: this.SIZE_X,
        PADDING_LEFT: this.PADDING_LEFT,
        PADDING_TOP: this.PADDING_TOP,
      });
    }
  }

  window.customElements.define('western-world', WesternWorld);
}
